var express  = require('express'),
    mongoose = require('mongoose');

var app = module.exports = express();

// Connect all of the things
mongoose.connect(process.env.MONGODB_URI || 'mongodb://127.0.0.1:27017/notification');

// Configure special environment for development
app.configure('development', function () {
  app.use(express.logger('dev'));
});

// Configure common application variables
app.set('port', process.env.PORT || 3000);

// Redirect to non-www
app.use(function (req, res, next) {
  var url  = req.url,
      host = req.headers.host;
  if (host.substr(0, 4) === 'www.') {
    return res.redirect(301, '//' + host.substr(4) + url);
  }
  next();
});

// Set all requests to respond with json by default, and enable CORS
app.use(function (req, res, next) {
  res.contentType('application/json');
  res.header('Access-Control-Allow-Origin', '*');
  res.header('Access-Control-Allow-Headers', 'X-Requested-With');
  next();
});

// Bootstrap the application
require('./support/passport')();

// Route middleware
app.use(express.responseTime());
app.use(express.bodyParser());
app.use(express.methodOverride());

// Require and mount all the related routes
require('fs').readdirSync(__dirname + '/routes').forEach(function (route) {
	if (route.charAt(0) === '.') { return; }
  route = route.replace(/\.js$/, '');
  app.use('/' + route, require(__dirname + '/routes/' + route));
});

app.use(require('./support/error-handler'));

var server = require('http').createServer(app);

server.listen(app.get('port'), function () {
  console.log('Server is now running on port ' + app.get('port'));
});
