var mongoose   = require('mongoose'),
    bcrypt     = require('bcrypt'),
    timestamps = require('mongoose-timestamp'),
    validate   = require('mongoose-validator').validate,
    Schema     = mongoose.Schema;

const SALT_WORK_FACTOR = 10;

var UserSchema = new Schema({
  email: {
    type:     String,
    trim:     true,
    required: true,
    index:    { unique: true },
    validate: [ validate('isEmail') ]
  },
  password: { type: String, required: true }
});

UserSchema.plugin(timestamps);

UserSchema.methods.comparePassword = function (password, next) {
  bcrypt.compare(password, this.password, function (err, match) {
    if (err) { return next(err); }
    next(null, match);
  });
};

UserSchema.statics.encryptPassword = function (password, next) {
  bcrypt.hash(password, SALT_WORK_FACTOR, function (err, hash) {
    if (err) { return next(err); }
    next(null, hash);
  });
};

var User = module.exports = mongoose.model('User', UserSchema);

UserSchema.pre('save', function (next) {
  var user = this;

  if (!user.isModified('password')) { return next(); }

  User.encryptPassword(user.password, function (err, hash) {
    user.password = hash; // Set the password to `undefined` even on failure
    next(err);
  });
});
