var http = require('http'),
    util = require('util');

/**
 * Create a new error to throw for use with the error handling middleware
 *
 * @param {String} message
 */
var Status = exports.Status = function (/* code, */ message) {
  var code = this.code = 500;

  if (typeof message === 'number') {
    code    = this.code = message;
    message = arguments[1];
  }

  this.message = message || http.STATUS_CODES[code];

  Error.call(this);
  Error.captureStackTrace(this, this.constructor);
};
util.inherits(Status, Error);
Status.prototype.name = 'Status';

/**
 * Create a new error to throw that represents a `404 - Not Found`
 *
 * @param {String} message
 */
var NotFound = exports.NotFound = function (message) {
  Status.call(this, 404, message);
};
util.inherits(NotFound, Status);
NotFound.prototype.name = 'NotFound';

/**
 * Create a new error to throw that represents a `400 - Bad Request`
 *
 * @param {String} message
 */
var BadRequest = exports.BadRequest = function (message) {
  Status.call(this, 400, message);
};
util.inherits(BadRequest, Status);
BadRequest.prototype.name = 'BadRequest';

/**
 * Create a new error to throw that represents a `405 - Method Not Allowed`
 *
 * @param {String} message
 */
var MethodNotAllowed = exports.MethodNotAllowed = function (message) {
  Status.call(this, 405, message);
};
util.inherits(MethodNotAllowed, Status);
MethodNotAllowed.prototype.name = 'MethodNotAllowed';

/**
 * Create a new error to throw that represents a `418 - I'm a Teapot`
 *
 * @param {String} message
 */
var ImATeapot = exports.ImATeapot = function (message) {
  Status.call(this, 418, message);
};
util.inherits(ImATeapot, Status);
ImATeapot.prototype.name = 'ImATeapot';

/**
 * Create a new error to throw that represents a `429 - Too Many Requests`
 *
 * @param {String} message
 */
var TooManyRequests = exports.TooManyRequests = function (message) {
  Status.call(this, 429, message);
};
util.inherits(TooManyRequests, Status);
TooManyRequests.prototype.name = 'TooManyRequests';

/**
 * Create a new error to throw that represents a `500 - Internal Server Error`
 *
 * @param {String} message
 */
var InternalServerError = exports.InternalServerError = function (message) {
  Status.call(this, 429, message);
};
util.inherits(InternalServerError, Status);
InternalServerError.prototype.name = 'InternalServerError';
