var test = require('./support/http');

describe('/users', function () {
  var email          = 'me@example.com',
      changeEmail    = 'change@example.com',
      password       = 'test',
      changePassword = 'change',
      id;

  it('should respond to POST but fail due to no params', function (done) {
    test('post', '/users').expect(400, done);
  });

  it('should respond to POST and respond with success code', function (done) {
    test('post', '/users')
      .send({ email: email, password: password })
      .expect(201)
      .end(function (err, res) {
        if (err) { return done(err); }

        id = res.body._id;

        return done();
      });
  });

  it('should not be able to create a user with the same email', function (done) {
    test('post', '/users').send({ email: email, password: password }).expect(400, done);
  });

  it('should not be able to create a user with a non email', function (done){
    test('post', '/users').send({ email: 'not', password: password }).expect(400, done);
  });

  it('should not be able to create a user with no email or password', function (done){
    test('post', '/users').send({}).expect(400, done);
  });

  it('should GET a user based on an id', function (done) {
    test('get', '/users/' + id).expect(200).end(function (err, res) {
      if (err) { return done(err); }

      var user = res.body;

      if (user.email === email && user._id === id) {
        return done();
      }
    });
  });

  it('should GET no user based on a non existing id', function (done) {
    test('get', '/users/123456789').expect(400, done);
  });

  it('should PUT a change to the users email', function (done) {
    test('put', '/users/' + id)
      .send({ email: changeEmail })
      .expect(200)
      .end(function (err, res) {
        if (err) { return done(err); }

        var user = res.body;

        if (user.email === changeEmail) {
          return done();
        }
      });
  });

  it('should change the users password on PUT', function (done) {
    test('put', '/users/' + id)
      .send({ password: changePassword })
      .expect(200)
      .end(function (err, res) {
        if (err) { return done(err); }

        if (res.body.password) {
          return done(new Error('Password shouldn\'t be sent to the user'));
        }

        done();
      });
  });

  it('should DELETE the user that was created', function (done) {
    test('del', '/users/' + id).expect(204, done);
  });

  it('should not DELETE a non existent user', function (done) {
    test('del', '/users/' + '123456789').expect(400, done);
  });

});
